# coding: utf-8
# pystacia/api/__init__.py
# Copyright (C) 2011 by Paweł Piotr Przeradowski
#
# This module is part of Pystacia and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

min_version = (6, 5, 9, 0)
name = 'MagickWand'
abis = (5, 4, 3, None)
